-module(kv_vnode_manager).

-behaviour(gen_server).

-export([
    start_link/0,
    get_vnode/1
]).

%%% gen_server callbacks
-export([init/1, handle_call/3, handle_cast/2]).

-record(kv_vnode_manager, {table :: ets:table()}).

-type state() :: #kv_vnode_manager{}.

%%% public api

start_link() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

get_vnode(Index) ->
    gen_server:call(?MODULE, {get_vnode, Index}).

%%% gen_server callbacks

-spec init([]) -> {ok, state()}.
init([]) ->
    {ok, #kv_vnode_manager{table = ets:new(?MODULE, [set])}}.

handle_call({get_vnode, Index}, _From, State) ->
    Table = State#kv_vnode_manager.table,
    Reply =
        case ets:lookup(Table, Index) of
            [{_, Pid}] ->
                {ok, Pid};
            [] ->
                {ok, Pid} = kv_vnode_sup:start_vnode(Index),
                ets:insert(Table, {Index, Pid}),
                {ok, Pid}
        end,
    {reply, Reply, State}.

handle_cast(_, _) -> error(not_implemented).
